<?php

namespace jrosset\CliProgram\Validation\Validators;

use DateTimeImmutable;
use DateTimeInterface;

/**
 * An argument/option value validator expecting a date and time
 *
 * @template-implements BasedValidator<RegexValidator>
 */
class DateTimeValidator extends AbstractDateTimeValidator {
    /**
     * @var bool Is the time part mandatory ?
     */
    private bool $timeMandatory;
    /**
     * @var DateTimeInterface The time part (if not mandatory)
     */
    private DateTimeInterface $timePart;

    /**
     * Create a validator
     *
     * @param bool                   $timeMandatory Is the time part mandatory ?
     * @param DateTimeInterface|null $timePart      The time part (if not mandatory)
     */
    public function __construct (bool $timeMandatory = false, ?DateTimeInterface $timePart = null) {
        $this->setTimeMandatory($timeMandatory);
        $this->setTimePart($timePart ?? (new DateTimeImmutable())->setTimestamp(0));
        parent::__construct();
    }

    /**
     * @inheritDoc
     */
    public function getValidDefault (mixed $default): string|bool|int|float|array|null {
        if ($default instanceof DateTimeInterface) {
            return $default->format('Y-m-d H:i:s');
        }
        return $default;
    }

    /**
     * @inheritDoc
     */
    protected function getPattern (): string {
        /** @noinspection RegExpUnnecessaryNonCapturingGroup */
        return /** @lang PhpRegExp */ '/^(?<year>\d{4})-(?<month>\d{2})-(?<day>\d{2})(?: (?<hour>\d{2}):(?<minute>\d{2})(?::(?<second>\d{2}))?)' . ($this->isTimeMandatory() ? '' : '?')
                                      . '$/';
    }

    /**
     * @inheritDoc
     */
    public function getValue (): DateTimeInterface {
        $value = new DateTimeImmutable();
        $value = $value->setDate(
            (int)$this->getInternalValidator()->getMatch('year'),
            (int)$this->getInternalValidator()->getMatch('month'),
            (int)$this->getInternalValidator()->getMatch('day'),
        );
        return $value->setTime(
            (int)($this->getInternalValidator()->getMatch('hour') ?? (int)$this->getTimePart()->format('%H')),
            (int)($this->getInternalValidator()->getMatch('minute') ?? (int)$this->getTimePart()->format('%i')),
            (int)($this->getInternalValidator()->getMatch('second') ?? (int)$this->getTimePart()->format('%s')),
        );
    }

    /**
     * Is the time part mandatory ?
     *
     * @return bool Is the time part mandatory ?
     */
    public function isTimeMandatory (): bool {
        return $this->timeMandatory;
    }
    /**
     * Is the time part mandatory ?
     *
     * @param bool $timeMandatory Is the time part mandatory ?
     *
     * @return $this
     */
    public function setTimeMandatory (bool $timeMandatory): self {
        $this->timeMandatory = $timeMandatory;
        return $this;
    }

    /**
     * The time part (if not mandatory)
     *
     * @return DateTimeInterface The time part (if not mandatory)
     */
    public function getTimePart (): DateTimeInterface {
        return $this->timePart;
    }
    /**
     * Set the time part (if not mandatory)
     *
     * @param DateTimeInterface $timePart The time part
     *
     * @return $this
     */
    public function setTimePart (DateTimeInterface $timePart): self {
        $this->timePart = $timePart;
        return $this;
    }
}